"use strict";
/**
 * Copyright (c) 2022, knokbak
 * All rights reserved.
 *
 * This file is subject to the terms and conditions of the BSD 3-Clause
 * License. A copy of the license can be found in the LICENSE file. If
 * a copy of the license was not distributed with this file, you can
 * obtain one at https://github.com/knokbak/cf-turnstile.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const DEFAULT_API_URL = 'https://challenges.cloudflare.com/turnstile/v0/siteverify';
function turnstile(secret, globalOptions) {
    if (secret && typeof secret !== 'string') {
        throw new TypeError('secret must be a string or undefined');
    }
    return async (token, options) => {
        options = {
            secret: options?.secret ?? globalOptions?.secret ?? secret,
            apiUrl: options?.apiUrl ?? globalOptions?.apiUrl ?? DEFAULT_API_URL,
            hostname: options?.hostname ?? globalOptions?.hostname,
            remoteip: options?.remoteip ?? globalOptions?.remoteip,
            action: options?.action ?? globalOptions?.action,
            cdata: options?.cdata ?? globalOptions?.cdata,
            debug: options?.debug ?? globalOptions?.debug ?? false,
            throwOnFailure: options?.throwOnFailure ?? globalOptions?.throwOnFailure ?? false,
        };
        if (!options.secret || typeof options.secret !== 'string') {
            throw new Error('options.secret is required (when a secret is not provided globally) and must be a string');
        }
        if (!token || typeof token !== 'string') {
            throw new Error('token is required and must be a string');
        }
        const formData = new URLSearchParams();
        formData.append('secret', options.secret);
        formData.append('response', token);
        const remoteip = options?.remoteip;
        if (remoteip) {
            formData.append('remoteip', remoteip);
        }
        if (options.debug) {
            console.log('cf-turnstile: sending request to', options.apiUrl);
            console.log('cf-turnstile: using form data:', formData);
        }
        const received = await sendRequest(options?.apiUrl ?? DEFAULT_API_URL, {
            method: 'POST',
            body: formData,
        }).then(x => x.json());
        const data = {
            success: received.success ?? false,
            timestamp: received.challenge_ts ? new Date(received.challenge_ts) : undefined,
            hostname: received.hostname,
            errors: received['error-codes'] ?? [],
            action: received.action,
            cdata: received.cdata,
        };
        data.errors = received['error-codes'] ?? [];
        if (data.errors.length > 0) {
            if (options.throwOnFailure) {
                throw new Error(`cf-turnstile: Cloudflare returned one or more error codes: ${data.errors.join(', ')}`);
            }
            return data;
        }
        if (!data.success) {
            if (options.throwOnFailure) {
                throw new Error(`cf-turnstile: Cloudflare could not verify the token`);
            }
            return data;
        }
        const hostname = options?.hostname;
        const action = options?.action;
        const cdata = options?.cdata;
        if (hostname && data.hostname !== hostname) {
            data.success = false;
            data.errors.push('cfts-hostname-mismatch');
            if (options.throwOnFailure) {
                throw new Error(`cf-turnstile: hostname mismatch: expected ${hostname}, got ${data.hostname}`);
            }
        }
        if (action && data.action !== action) {
            data.success = false;
            data.errors.push('cfts-action-mismatch');
            if (options.throwOnFailure) {
                throw new Error(`cf-turnstile: action mismatch: expected ${action}, got ${data.action}`);
            }
        }
        if (cdata && data.cdata !== cdata) {
            data.success = false;
            data.errors.push('cfts-cdata-mismatch');
            if (options.throwOnFailure) {
                throw new Error(`cf-turnstile: cdata mismatch: expected ${cdata}, got ${data.cdata}`);
            }
        }
        return data;
    };
}
;
function sendRequest(url, options) {
    try {
        return fetch(url, options);
    }
    catch {
        const fetch = require('node-fetch');
        return fetch(url, options);
    }
}
;
module.exports = turnstile;
exports.default = turnstile;
//# sourceMappingURL=index.js.map